#include "action.h"
#include "errors.h"
#include "gui/common/session.h"

#include <stdlib.h>
#include <string.h>



/******************************************************************************/
/* context-menu helpers */



/**
 * destroy a file-entry in a menu-action context
 * @param af  the file-entry
 */
void action_file_dst(action_file *af) {
  if(af) {
    if(af->tag)
      free(af->tag);
    if(af->md)
      meta_del_all(&af->md);
    free(af); }}



/**
 * add a file-entry to a menu-action context
 * @param ac   an action context to add to, or NULL
 * @param tag  the file's name
 */
action_file *action_file_new(action_context *ac,char *tag) {
  action_file *af;
  if((af=malloc(sizeof(action_file)))) {
    memset(af,0,sizeof(action_file));
    if(!(af->tag=strdup(tag))) {
      free(af);
      goto done; }
    if(ac) {
      action_file *ar=ac->selection;
      if(!ar)
	ac->selection=af;
      else {
	while(ar->next)
	  ar=ar->next;
	ar->next=af; }}}
 done:
  return af; }



/**
 * destroy an action-context
 * @param ac  the action-context to destroy
 */
void action_context_dst(action_context *ac) {
  action_file *af;
  if(ac) {
    if(ac->cwd)
      free(ac->cwd);
    action_file_dst(ac->single);
    while((af=ac->selection)) {
      ac->selection=ac->selection->next;
      action_file_dst(af); }
    free(ac); }}



/**
 * create a new action-context
 * @return an action-context. serial is that of the calling session.
 * @param serial  the serial of the calling session
 * @param cwd     the work-directory
 */
action_context *action_context_new(long serial,char *cwd) {
  action_context *ac;
  if(!(ac=malloc(sizeof(action_context))))
    return NULL;
  memset(ac,0,(sizeof(action_context)));
  ac->serial=serial;
  ac->flags=AF_LAST; /* assuming only one entry (then it'll be the First and Last) */
  if(cwd&&!(ac->cwd=strdup(cwd))) {
    free(ac);
    return NULL; }
  return ac; }



/**
 * create a new action-context, preset with this session's selection
 * @return an action-context. serial is that of the calling session.
 * @param ns    the session
 * @param file  
 */
action_context *action_context_from_selection(void *ns,char *file) {
  action_context *ac=NULL;
  action_file    *af=NULL;
  char           *sel=NULL;
  int             c=0;

  if(!ns||!(sel=noir_sess_get_selection(ns)))
    return NULL;

  if(!file||!strcmp(file,sel)||(selection_contains(ns,file)!=NOIR_SUCC))
    return NULL;

  if(!(sel=strdup(sel)))
    return NULL;

  if((ac=action_context_new(noir_sess_get_serial(ns),noir_sess_get_cwd(ns)))) {
    if(file&&!(af=action_file_new(NULL,file))) {
      action_context_dst(ac);
      ac=NULL; }
    else {
      char *p=sel,*b=sel;

      ac->single=af;

      while((p=strchr(p,'\n'))) {
        c++;
        *p='\0';
        action_file_new(ac,b);
        b=(++p); }

      action_file_new(ac,b); }}

  ac->flags|=AF_SELECTION; /* we have more than one file, set "selection" flag */
  if(c<1)
    ac->flags|=AF_LAST;    /* we have more than one file, clear "last" flag */

  free(sel);
  return ac; }
