/** @file  action.h
 *  @brief action-contexts for context-menu ("right click") actions
 */

#ifndef ACTION_H
#  define ACTION_H

#  include "meta.h"

/** context for an action selected from the context-menu.
 * this is interesting mostly when the action is to be applied to a
 * seletion of several objects. */
typedef enum {
  AF_NONE=0,           /**< no flags */
  AF_YES_TO_ALL=1,     /**< "yes to all", stop asking */
  AF_CANCEL_ALL=2,     /**< "no to all",  stop asking */
  AF_SELECTION=4,      /**< selection set on action-context (multiple files) */
  AF_DONE=8,           /**< terminate menu-loop */
  AF_LAST=16           /**< last item to process, free any resources. plugins using this should check this last thing before returning (specifically, after any calls to form_yes_no_all()) */
} action_flags;


/** a file in an action-context
 */
typedef struct _action_file {
  char                *tag;   /**< the file's tag (path, UUID, URL, ...) */
  meta                *md;    /**< the file's meta-data */
  struct _action_file *next;  /**< next file in list (for selection only) */
} action_file;


/** an action-context for context-menu ("right click") actions
 */
typedef struct _action_context {
  action_flags  flags;        /**< flags */
  char         *cwd;          /**< directory the selected files live in */
  action_file  *selection;    /**< selection (linked list), if any */
  action_file  *single;       /**< file that was right-clicked on */
  long          serial;       /**< serial no. of the session */
} action_context;



void            action_file_dst(action_file *);
action_file    *action_file_new(action_context *ac,char *tag);
void            action_context_dst(action_context *ac);
action_context *action_context_new(long serial,char *cwd);
action_context *action_context_from_selection(void *ns,char *file);

#endif
