/** @file  delete.c
 *  @brief context-menu action "delete"
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gui/common/alert.h"      /* form_*() */
#include "gui/common/session.h"    /* noir_sess_*() */
#include "file.h"       /* file_job_create() */
#include "meta.h"       /* struct _meta */
#include "util.h"       /* estring_*() */
#include "action.h"       /* action_file_dst() */


#define TMPL "Really delete\n%s?"



/******************************************************************************/



/**
 * context-menu action: delete a file
 * @return an error-code
 * @param serial    the serial number of the session that called us
 * @param filename  the file to act on
 * @param md        the file's meta-data
 * @param ac        the context of this action (selection et al.)
 */
int efun(long serial,char *filename,meta **md,action_context *ac) {
  noir_sess   *ns=noir_sess_get_by_serial(serial);
  int          ret=NOIR_IGNORED,cnt=0;
  estring     *msg=estring_new(2048),*files=estring_new(2048);
  action_file *sf=ac->selection,*prv=NULL,*curr=NULL;

  if(!filename)
    return NOIR_FAIL;

  if(!msg)
    return NOIR_OOM;

  if(!strcmp(filename,DIR_DELIM_STR))
    return NOIR_IGNORED;

  estring_printf(msg,TMPL,filename);
  if(ESTRING_GET_CSTRING(msg)&&sf&&(sf->next||(strcmp(sf->tag,filename)))) {
    estring_appendf(msg,"\n\nRemaining files:");
    while(sf) {
      estring_appendf(files,"%s%s",(cnt++>0)?"\n":"",sf->tag);
      if(strcmp(sf->tag,filename)) {
        estring_appendf(msg,"\n%s",sf->tag);
     /* E(0,"\"%s\" => \"%s\"\n",sf->tag,ESTRING_GET_CSTRING(msg)); */
        prv=sf; }
      else {
        curr=sf;
        if(prv)
          prv->next=sf->next;
        else
          ac->selection=sf->next; }
      sf=sf->next; }}

  if(!ESTRING_GET_CSTRING(msg))
    return NOIR_OOM;

  if(form_yes_no_all(ns,ESTRING_GET_CSTRING(msg),ac)==NOIR_SUCC) {
    if(ac->flags&AF_YES_TO_ALL) {
      ret=(file_job_create(ns,NOIR_JOB_RM,ESTRING_GET_CSTRING(files),NULL,NULL)>=0)?NOIR_SUCC:NOIR_FAIL;
      while((sf=ac->selection)) {
        ac->selection=sf->next;
        action_file_dst(sf); }
      curr=NULL; }
    else
      ret=(file_job_create(ns,NOIR_JOB_RM,filename,NULL,NULL)>=0)?NOIR_SUCC:NOIR_FAIL; }

  estring_free(msg,TRUE);
  estring_free(files,TRUE);

  if(curr)
    action_file_dst(curr);

  return ret; }



/******************************************************************************/
