/** @file  actions/thumbnail/thumbnail.c
 *  @brief rebuild thumbnail(s)
 */

#include <stdio.h>

#include "config.h"

#include "errors.h"
#include "debug.h"
#include "action.h"
#include "meta.h"
#include "model.h"

#include "thumbnailer/thumbnail.h"
#include "gui/common/alert.h"



/*****************************************************************************/



/**
 * re-generate thumbnail for the given item
 */
static int rethumb(file_info *fi) {
  int ret=NOIR_IGNORED;
  if(!fi||!fi->tag)
    E(0,"fi: %p, tag: \"%s\"\n",fi,fi?fi->tag:"???");
  else if(((ret=thumbnail_disk_gc_one(fi->tag))==NOIR_SUCC)||
          (ret==NOIR_IGNORED)) {
    ret=thumbnail_enqueue(fi);
    if(ret==NOIR_SUCC)  /* instant kama-sutra. can happen for folder-icons. */
      model_evt(fi,MODEL_UPD,FF_THUMB);
    else if(ret==NOIR_PARTIAL)  /* request to re-gen thumbnail enqueued. */
      fi->have&=(~FF_THUMB);
    else if(ret==NOIR_TYPEMISMATCH)
      form_alert(NULL,"Sorry, don't know how to thumbnail\n\"%s\"...",fi->tag);
    else {
      E(0,"enqueue failed for \"%s\": %d\n",fi->tag,ret);
      fi->have&=(~FF_THUMB); }}
  else
    E(0,"could not delete thumbnail(s) for \"%s\" => %d\n",fi->tag,ret);
  return ret; }



/**
 * context-menu action: rethumbnail items
 * @return an error-code
 * @param serial    the serial number of the session that called us
 * @param filename  the file to act on
 * @param md        the file's meta-data
 * @param ac        the context of this action (selection et al.)
 */
int efun(long serial,char *filename,meta **md,action_context *ac) {
  file_info *fi=file_info_fetch(NULL,filename);

  if(fi) {
    rethumb(fi);

    fi=fi->child;
    while(fi) {
      rethumb(fi);
      fi=fi->next; }

    return NOIR_SUCC; }

  return NOIR_NODATA; }



/*****************************************************************************/
