/** @file  session_private.h
 *  @brief the session-struct.  one per window.  a session may hold several views.
 */

#ifndef SESSION_PRIVATE_H
#  define SESSION_PRIVATE_H
#  include <gtk/gtk.h>
#  include "evidence.h"

#  ifdef HAVE_PCRE
#    include <pcre.h>
#  endif



/** a session */
struct _noir_sess {
  long          serial;       /**< "primary key" : ) */
  unsigned long focused;      /**< when was this window last focused? */

  client_funs *clients;       /**< list of views in this window */
  client_funs *client;        /**< current view in this window */

  GtkWidget *window;          /**< top window */
  GtkWidget *tooltip_window;  /**< window for GTK-tooltip */

  GtkWidget *notebook;        /**< the thing with the tabs */
  GtkWidget *container;       /**< window's top child: paned if there is a shelf, notebook otherwise */
  GtkWidget *about;           /**< about "view" */

  GtkWidget *tooltip_vbox,    /**< child of tooltip_window, container for label & icon */
            *tooltip_label,   /**< info-text for tooltip */
            *tooltip_icon,    /**< icon for tooltip */
            *tooltip_origin;  /**< Evas-mode: the widget (shelf/icon-view) the tooltip is related to */
  gint   tooltip_x,tooltip_y; /**< desired location for tooltip (root-cordinates) */

  GdkPixbuf *drag_icon;       /**< DND-icon */

  void  *tt_evas;             /**< evas: object containing tooltip */

  size_t typebuffer_max;      /**< maximum input length for type-buffer */

  int sortflags;              /**< sort-hints: unsorted, folders first, ... */

  int thumbnails_in_iconlist,/**< show thumbnails rather than icons in icon-view? */
      thumbnails_in_shelf,   /**< show thumbnails rather than icons in shelf? */
      without_notebook,      /**< hide notebook-tabs? */
      without_shelf,         /**< disable "shelf"? */
      without_decorations,   /**< disable window-borders? */
      with_tooltips,         /**< mouse-over + delay tooltips? (can still be activated
                                  by key-press/mouse-clicke otherwise) */
      sema_focus,            /**< "semaphore" */
      typebuffer_open,       /**< are busy entering text? */
      typebuffer_entered,    /**< tab-completing: how many chars were typed? */
      start_in,              /**< which view to show first */
      fullscreen,            /**< maximize window? */
      icon_view_last_tooltip;/**< GTK: index of last icon we made a tooltip for */

  char *pcre,                /**< last regex, as clear-text */
       *evas_engine,         /**< use OpenGL rather than straight X? */
       *geometry,            /**< geometry string */
       *cwd,                 /**< current work-dir */
       *pwd,                 /**< previous work-dir */
       *gdb,                 /* get-dir base (ugly hack must go ###) */
       *typebuffer,          /**< input-buffer for our "microShell" */
       *tabbuffer,           /**< input-buffer last tab pattern; this is used only when the pattern actually is a glob/regex (as in a shell's tab-completion), as in this case, the resulting typebuffer (match) could not be tabbed again */
       *tooltip,             /**< info-text for tooltip */
       *cfg_file,            /**< setup-file (forthcoming) */
       *selection,           /**< currently selected files, \\n separated */
       *drag_files;          /**< which files were last dragged *from* ev */

  file_info  *cd;            /**< current dir (file-info) */

  char       *iconset;       /**< option takes path or name, will set path+file */
  void       *theme;         /**< theme data */

#  ifdef HAVE_PCRE
  pcre       *re;            /**< typebuffer: regex */
  pcre_extra *pe;            /**< typebuffer: regex */
#  else
  void       *re;            /**< binary padding so ABI stays consistent */
  void       *pe;            /**< binary padding so ABI stays consistent */
#  endif
  void       *tb;            /**< evas: type-buffer object(s) */

  GdkDragContext *drag_context;  /**< drag-context */

  struct _noir_sess *next;   /**< next session in list */
};

#endif
