/* theming of evas-based icon-list: read theme-setup from file,
   return an initiliazed theme-struct. */

#include "config.h"

#include "gui/common/session_private.h"

#include "theme.h"

#include "setup.h"
#include "util.h"          /* timeout_*() */
#include "mime.h"          /* MIME_* */
#ifdef HAVE_EVAS2
#  include "evas_macros.h"                      /* get_evas_from_iconlist() */
#  include "gui/gtk2/icon_view/gevas_bind.h"    /* ICON_LIST_GET_FROM_WIDGET() */
#endif

#include "gui/gtk2/common/shelf.h"         /* CLIENT_SHELF */

#include "file.h"

#include "icon_path.h"     /* icon_path_get() */

#include <stdlib.h>        /* atof() */
#include <string.h>
#include <dlfcn.h>
#include <ctype.h>
#include <sys/stat.h>      /* S_ISDIR() */



#define LISTENER "theme{evas}"

typedef enum { THEME_ODA, THEME_OFA, THEME_OAA } theme_tag_type;



typedef enum {
  FNT_NONE = 0,    /**< no fonts */
  FNT_TB   = 1,    /**< typebuffer-font */
  FNT_TT   = 2,    /**< tooltip-font */
  FNT_ICN  = 4,    /**< icon-label font */
  FNT_TP   = 8,    /**< text-path font */
  FNT_ALL  = 15    /**< all fonts */
} fonts;

typedef enum {
  PT_SB  =     1,  /**< scrollbar factory */
  PT_TT  =     2,  /**< tooltip factory */
  PT_TB  =     4,  /**< typebuffer engine */
  PT_RB  =     8,  /**< rubber-band factory */
  PT_BG  =  0x10,  /**< background engine */
  PT_IC  =  0x20,  /**< icon-factory */
  PT_SH  =  0x40,  /**< icon-view (background data) */
  PT_IV  =  0x80,  /**< shelf (background data) */
  PT_ID  = 0x100,  /**< idle-handler (optional, hence not part of PT_ALL) */
  PT_IX  = 0x200,  /**< extension */
  PT_ALL =  0xff   /**< "all of the above" */
} parse_theme;







/*****************************************************************************/



/**
 * get icon dimensions
 * @return an error-code
 * @param t - the theme
 * @param w - where to store icon-width,  NULL for don't
 * @param h - where to store icon-height, NULL for don't
 */
int theme_get_iconsize(const noir_theme *t,int *w,int *h) {
  int ret=NOIR_IGNORED;
  if(!t)
    return NOIR_FAIL;
  if(w) {
    if((*w=t->icon_width)<=0)
      ret=NOIR_NODATA;
    else
      ret=NOIR_SUCC; }
  if(h) {
    if((*h=t->icon_height)<=0)
      ret=NOIR_NODATA;
    else if(ret==NOIR_NODATA)
      ret=NOIR_PARTIAL;
    else
      ret=NOIR_SUCC; }
  return ret; }



/**
 * get icon-mode
 * @return the icon-mode
 * @param t - the theme
 */
ic_mode theme_get_iconmode(const noir_theme *t) {
  return t?t->icon_mode:IC_NONE; }






/*****************************************************************************/
/* saving themes */



#define SAVE_AS "/" DIR_USER "/" THEME_CFG

/**
 * theme_dump - save a setup to a plaintext file.
 * @what - the session to save
 */
int theme_dump(const void *what) {
  noir_sess  *ns=(noir_sess *)what;
  noir_theme *t=((noir_theme *)noir_sess_get_theme(ns));
  FILE       *fh;
  char       *fn,*p,*v;
  size_t      l=strlen(home)+sizeof(SAVE_AS);
  int         ret=NOIR_FAIL;

  if(!(fn=malloc(l)))
    return NOIR_OOM;

  strcat(strcpy(fn,home),SAVE_AS);

  E(0,"saving theme as \"%s\"...\n",fn);

  if((fh=fopen(fn,"wb"))) {
    noir_theme_el *e=t->data;
    ret=NOIR_SUCC;

/* #define SAVE_FMT "%22.22s" */
    while(e) {
      v=NULL;
      switch(e->type) {
        case ET_NIL:
          break;
        case  ET_INT:
          esprintf(&v,"%ld",e->val);
          break;
        case  ET_FLT:
          esprintf(&v,"%ld.%ld",theme_el_get_float_int(e),abs(theme_el_get_float_decimals(e)));
          break;
        case ET_STR:
          esprintf(&v,"%s",e->ptr?e->ptr:"");
          break;
        case ET_RGB:
          esprintf(&v,"#%06lx",e->val);
          break;
        case ET_PTR:
          esprintf(&v,"%p",e->ptr);
          break; }

      if(v) {
        fprintf(fh,"%s%s: %s\n",
                ((e->type==ET_PTR)||!(e->flags&EF_MODIFIED))?"; ":"",
                e->key,v);
        free(v); }
      e=e->next; }

    if(ns->iconset) {
      if(strstr(ns->iconset,PROTOCOL_DELIMITER)||(ns->iconset[0]==DIR_DELIMITER))
        p=ns->iconset;
      else if((p=strrchr(ns->iconset,DIR_DELIMITER)))
        p++;
      if(p)
        fprintf(fh,"%s: %s\n","iconset",p); }

    { int c=0,m=t->icon_mode;
      char *im[]={ "none", "w", "h", "{w,h}/w", "{w,h}/h", "auto", "fill", NULL, NULL };
      while(m&&im[c]) {
     /* printf("%d %d ",c,m); printf("%s\n",im[c]); */
        c++;
        m=m>>1; }
      if(im[c])
        fprintf(fh,"%s: %s\n","iconmode",im[c]); }

    fclose(fh); }
  free(fn);
  return ret; }






/*****************************************************************************/
/* helpers */



/**
 * theme_new - create a new theme struct.
 * @ns - the session to create the theme for
 */
noir_theme *theme_new(noir_sess *ns) {
  noir_theme *t;
  if((t=malloc(sizeof(noir_theme)))) {
    memset(t,0,sizeof(noir_theme));
    t->session=ns;
    t->icon_mode=IC_AUTO;
#if 0
    t->icon_width=ICON_SIZE_IN_LIST;
    t->icon_height=(int)((ICON_SIZE_IN_LIST*3)/4);
#endif
#ifdef WITH_DOT_DIR
    t->tf|=TF_DOTDIR;
#endif
  }
  return t; }



char *theme_ms(const noir_theme *t,const char *path,const char *dir,const char *prefix,const char *file,const char *suffix) {
  char *r=NULL,*a=NULL,*b=NULL;

  esprintf(&r,"%s/%s/%s%s%s",path?path:"",dir?dir:"",prefix?prefix:"",
                             file?file:"",suffix?suffix:"");

  if(r) {
    if(!file_exists(r)) {
      /* not found in typed dir in same path as theme-dir, try theme-dir */
      if(t) {
        if(strcmp(t->name,dir)) /* try in theme-dir parallel to this dir */
          if(!(a=theme_ms(NULL,path,t->name,prefix,file,suffix))) {
            /* try in main theme-dir */
            if((b=get_setup_file(DIR_THEME))) {
              if(strcmp(b,path)) {
                a=theme_ms(t,b,dir,prefix,file,suffix); }
              free(b); }}}
      free(r);
      r=a;
      if(dir&&!strcmp(dir,"engines")&&!file_exists(r)) {
        esprintf(&r,PACKAGE_PREFIX "/lib/evidence/gui/efl/engines/%s%s%s",prefix?prefix:"",file?file:"",suffix?suffix:"");
        if(!r)
          return NULL;
        else if(!file_exists(r)) {
          free(r);
          r=NULL; }}}}

  return r; }



#ifdef HAVE_EVAS2
int theme_ani_get(noir_theme *t,const char *name,void **l,void (**f)(icon_list *,icon *)) {
  void *fun=NULL,*lib=NULL;
  char *p=theme_el_get_string(&t->data,name),*x=NULL;
  int   ret=NOIR_IGNORED;

  if((x=theme_ms(t,t->path,"engines","ia_",p,".so"))&&
     (lib=dlopen(x,RTLD_NOW))) {
    if(!(fun=dlsym(lib,"animation"))) {
      ret=NOIR_NOTFOUND;
      dlclose(lib);
      lib=NULL; }
    else
      ret=NOIR_SUCC;
    free(x); }
  else
    ret=NOIR_NOFILE;

  if(l)
    *l=lib;
  if(f)
    *f=fun;

  return ret; }



static inline int theme_font_chk(Evas *e,char *font,char *desc) {
  Evas_Object *o;
  Evas_Coord   x,y,w=0,h=0;

  if(!font)
    font="<none>";
  if(!desc)
    desc="unspecified font";

  if(!e) {
    E(0,"could not create a text-object with %s without an evas-canvas!\n",desc);
    return NOIR_FAIL; }

  if((o=evas_object_text_add(e))) {
    evas_object_text_font_set(o,font,40);
    evas_object_text_text_set(o,"abcABC123-_.");
    evas_object_geometry_get(o,&x,&y,&w,&h);
    evas_object_del(o);
    if(w*h)
      return NOIR_SUCC; }
  else {
    E(0,"could not create a text-object with %s on evas at %p!?\n",desc,e);
    return NOIR_FAIL; }

  if(file_exists(font))
    E(0,"could not create a text-object with %s "
      "(\"%s\", requested in \"evidence.theme\"); the font "
      "or your version of freetype may be broken. (size %.0fx%.0f)\n\n",desc,font,(float)w,(float)h);
  else
    E(0,"could not create a text-object with %s "
      "(\"%s\", requested in \"evidence.theme\"); it doesn't exist.\n\n",desc,font);
  return NOIR_FAIL; }



static int font_fallback(noir_theme *t,char *font,char *key,int got) {
  if(!got) {
    E(0,"%s falling back on font \"%s\" -- things"
                       " will not look the way they should...\n",key,font);
    theme_el_set(&t->data,key,font); }
  return NOIR_IGNORED; }



char *theme_font_find(const noir_theme *t,const char *v) {
  char *f;
  if(!(f=theme_ms(t,t->path,"fonts","",v,".ttf"))) {
    if(!(f=(char *)get_setup_file(v))) {
      char   *fds[]={ "/usr/X11R6/lib/X11/fonts",
                      "/usr/X11R6/lib/X11/fonts/truetype",
                      "/usr/X11R6/lib/X11/fonts/TrueType",
                      "/usr/X11R6/lib/X11/fonts/msttcorefonts",
                      "/usr/share/fonts",
                      "/usr/share/fonts/truetype",
                      NULL };
      char   *fd;
      int     c=0;
      size_t  l2,l=4+2+strlen(v);

      while(!f&&(fd=fds[c++])) {
        if((f=malloc(l2=(strlen(fd)+l)))) {
          snprintf(f,l2,"%s/%s.ttf",fd,v);
          if(!file_exists(f)) {
            fd=&f[strlen(f)-3];
            strcpy(fd,"TTF");
            if(!file_exists(f)) {
              free(f);
              f=NULL; }}}}

      if(!f)
        f=strdup(v); }}

  if(debug>=4) {
    if(!strcmp(v,f))
      E(0,"font \"%s\" found...\n",v);
    else
      E(0,"font \"%s\" found as \"%s\"...\n",v,f); }
  return f; }






/*****************************************************************************/
/* idle-handlers  */



int theme_idle_start(noir_sess *ns) {
  Evas          *e=NULL;
  noir_theme    *t;
  icon_list     *il=NULL;
  noir_iconview *iv=(noir_iconview *)noir_sess_client_get_data(ns,CLIENT_ICONVIEW);
  noir_shelf    *sv=(noir_shelf *)noir_sess_client_get_data(ns,CLIENT_SHELF);
  void          *h;

  if(!ns||!(t=((noir_theme *)noir_sess_get_theme(ns)))) {
    E(0,"(%ld): no theme!? (%p)\n",ns?ns->serial:0,ns?((noir_theme *)noir_sess_get_theme(ns)):NULL);
    return NOIR_FAIL; }

  if(sv&&(il=ICON_LIST_GET_FROM_WIDGET(sv->shelf_view))&&il->canvas) {
    e=get_evas_from_iconlist(il);
    if(t->idle_init_shelf&&(t->idle_freq_shelf=t->idle_init_shelf(il,&(t->idle_data_shelf),t)))
      t->idle_hndl_shelf=timeout_add(t->idle_freq_shelf,(int (*)(void *))t->idle_func_shelf,&(t->idle_data_shelf)); }

  if(iv&&(il=ICON_LIST_GET_FROM_WIDGET(iv->icon_view))&&il->canvas) {
    e=get_evas_from_iconlist(il);
    if(t->idle_init_iconview&&(t->idle_freq_iconview=t->idle_init_iconview(il,&(t->idle_data_iconview),t)))
      t->idle_hndl_iconview=timeout_add(t->idle_freq_iconview,(int (*)(void *))t->idle_func_iconview,&(t->idle_data_iconview)); }

#if 1
  if(il) {
    h=t->rubber_new(il);
    t->rubber_dst(&h);
    h=t->tb_set(il,NULL,"");
    t->tb_dst(&h); }
  else
    E(0,"could not initialize rubber-band and type-buffer listeners!?\n");
#endif

  if(e) {
    int    w=0,h=0;
    char  *file,*r,
          *fallback=NULL;
    fonts  got=FNT_NONE;

    /* default icon-size */
    if((file=icon_path_get(NULL,MIME_FOLDER))||
       (file=icon_path_get(NULL,"unknown"))) {
      Evas_Object *o=evas_object_image_add(e);
      evas_object_image_file_set(o,file,NULL);
      evas_object_image_size_get(o,&w,&h);
      evas_object_del(o); }
    t->icon_width=w?w:ICON_SIZE_IN_LIST;
    t->icon_height=h?h:(int)((ICON_SIZE_IN_LIST*3)/4);

    if(theme_font_chk(e,(r=theme_el_get_string(&t->data,"typebufferfont")),"typebufferfont")==NOIR_SUCC) {
      fallback=r; got|=FNT_TB; }
    if(theme_font_chk(e,(r=theme_el_get_string(&t->data,"tooltipfont")),"tooltipfont")==NOIR_SUCC) {
      fallback=r; got|=FNT_TT; }
    if(theme_font_chk(e,(r=theme_el_get_string(&t->data,"iconfont")),"iconfont")==NOIR_SUCC) {
      fallback=r; got|=FNT_ICN; }
    if(theme_font_chk(e,(r=theme_el_get_string(&t->data,"textpathfont")),"textpathfont")==NOIR_SUCC) {
      fallback=r; got|=FNT_TP; }

    if(got!=FNT_ALL) {
      if(fallback) {
        font_fallback(t,fallback,"iconfont",      got&FNT_ICN);
        font_fallback(t,fallback,"tooltipfont",   got&FNT_TT);
        font_fallback(t,fallback,"textpathfont",  got&FNT_TP);
        font_fallback(t,fallback,"typebufferfont",got&FNT_TB); }
      else
        E(0,"no fonts found, icon-list will look very broken...\n"); }}

  return NOIR_SUCC; }



int theme_idle_destall(noir_sess *ns) {
  Evas          *e=NULL;
  noir_theme    *t;
  icon_list     *il;
  int            remove,ret=NOIR_SUCC;
  noir_iconview *iv=(noir_iconview *)noir_sess_client_get_data(ns,CLIENT_ICONVIEW);
  noir_shelf    *sv=(noir_shelf *)noir_sess_client_get_data(ns,CLIENT_SHELF);

  if(!ns||!(t=((noir_theme *)noir_sess_get_theme(ns))))
    return NOIR_NODATA;

  /* destall shelf's idle-handler */
  if(sv&&sv->shelf_view&&(il=ICON_LIST_GET_FROM_WIDGET(sv->shelf_view))&&il->canvas) {
    e=get_evas_from_iconlist(il);
    if(t->idle_hndl_shelf) {
      timeout_remove(&t->idle_hndl_shelf);  /* remove idler even if cleanup doesn't work */
      remove=0;
      if(t->idle_exit_shelf) {
        if(t->idle_exit_shelf(&(t->idle_data_shelf)))
          remove=1; }
      else
        remove=1;
      if(remove) {
        E(2,"destalling idle-handler for shelf.\n");
        dlclose(t->idle_shelf_lib);
        t->idle_shelf_lib=NULL; }
      else {
        ret=NOIR_FAIL;
        E(0,"idle-handler for shelf would not destall!?\n"); }}}

#if 0
  il=iv?ICON_LIST_GET_FROM_WIDGET(iv->icon_view):NULL;
  E(0,"B  %p, %p, %p, %p\n",iv,iv?iv->icon_view:NULL,il,il?il->canvas:NULL);
#endif

  /* destall icon-view's idle-handler */
  if(iv&&iv->icon_view&&(il=ICON_LIST_GET_FROM_WIDGET(iv->icon_view))&&il->canvas) {
    e=get_evas_from_iconlist(il);
    if(t->idle_hndl_iconview) {
      timeout_remove(&t->idle_hndl_iconview);  /* remove idler even if cleanup doesn't work */
      remove=0;
      if(t->idle_exit_iconview) {
        if(t->idle_exit_iconview(&(t->idle_data_iconview))) {
          remove=1; }}
      else
        remove=1;
      if(remove) {
        E(2,"destalling idle-handler for icon-view.\n");
        dlclose(t->idle_iconview_lib);
        t->idle_iconview_lib=NULL; }
      else {
        ret=NOIR_FAIL;
        E(0,"idle-handler for icon-view would not destall!?\n"); }}}

  return ret; }



static int theme_idle_install(noir_sess *ns,char *k,char *v,size_t l) {
  noir_theme  *t=((noir_theme *)noir_sess_get_theme(ns));
  int          ret=NOIR_FAIL;
  char        *x;
  void        *lib;

  int        (*idle_init)(void *,void **,struct _noir_theme *);
  int        (*idle_exit)(void **);
  int        (*idle_func)(void **);

  if(((ret=dlmulti(v,x=theme_ms(t,t->path,"engines","id_",v,".so"),RTLD_NOW,
                   &lib,"!idle_func !idle_init ?idle_exit",
                   &idle_func,&idle_init,&idle_exit))==NOIR_SUCC)||
     (ret==NOIR_PARTIAL)) {

    if(theme_idle_destall(ns)==NOIR_FAIL)
      goto done;

    ret=NOIR_SUCC;

    if(!strcmp(k+l-5,"shelf")) {
      t->idle_shelf_lib=lib;
      t->idle_init_shelf=idle_init;
      t->idle_exit_shelf=idle_exit;
      t->idle_func_shelf=idle_func; }
    else if(!strcmp(k+l-8,"iconview")||!strcmp(k+l-4,"icon")) {
      t->idle_iconview_lib=lib;
      t->idle_init_iconview=idle_init;
      t->idle_exit_iconview=idle_exit;
      t->idle_func_iconview=idle_func; }
    else {
      ret=NOIR_IGNORED;
      E(0,"unknown configuration key %s ignored...\n",k); }}

 done:
  if(x)
    free(x);

  return ret; }



/*****************************************************************************/



noir_theme_el *et_get_els_for(noir_theme *t,char *engine) {
  return NULL; }



int theme_engine_scan(char *engine) {
  noir_theme    *t=theme_new(NULL);
  noir_theme_el *e,*f;
  int            ret=NOIR_SUCC;

  if(!t)
    return NOIR_OOM;

  if(!(e=et_get_els_for(t,engine))) {
    E(1,"failed to load \"%s\"...  (Azundris-only message :)\n",engine);
    ret=NOIR_FAIL; }
  else {
    for(f=e;f;f=e) {
      if(f->flags&EF_BOUNDS) {
        if(f->type==ET_INT)
          printf("  %s{%s}: %ld - %ld\n",f->key,theme_el_get_type(f),f->lo,f->hi);
        else if(f->type==ET_FLT)
          printf("  %s{%s}: %lf - %lf\n",f->key,theme_el_get_type(f),((float)f->lo)/NOIR_THEME_FLOAT_PRECISION,((float)f->hi)/NOIR_THEME_FLOAT_PRECISION);
        else
          printf("  %s{%s}: <unknown bounds type>\n",f->key,theme_el_get_type(f)); }
      else
        printf("  %s{%s}\n",f->key,theme_el_get_type(f));
      e=f->next;
      theme_el_dst(&t->data,f); }}

  free(t);

  return ret; }



/**
 * listener for string properties
 * @return an error-code
 * @param key   the name of the property
 * @param type  the data-type of the property (string? : )
 * @param tag   the numeric tag of the property
 * @param data  the icon-list
 * @param te    linked list of theme-elements
 */
static int theme_string_listener(const char *key,const noir_theme_el_type type,const int tag,void *data,noir_theme_el **te) {
  char          *val=theme_el_get_string(te,key);
  noir_sess     *ns=(noir_sess *)data;
  noir_theme    *t=(noir_theme *)((noir_theme *)noir_sess_get_theme(ns));
#if 0
  noir_iconview *iv=(noir_iconview *)noir_sess_client_get_data(ns,CLIENT_ICONVIEW);
  icon_list     *il=ICON_LIST_GET_FROM_WIDGET(iv->icon_view);
#endif

  if(val) {
    void (**fun)(icon_list *,icon *)=NULL;
    void  **lib=NULL;

    E(1,"%s <- %s\n",key,val);

    switch(tag) {
      case THEME_ODA: fun=&t->ani_open_folder; lib=&t->ani_open_folder_lib; break;
      case THEME_OFA: fun=&t->ani_open_file;   lib=&t->ani_open_file_lib;   break;
      case THEME_OAA: fun=&t->ani_open_app;    lib=&t->ani_open_app_lib;    break;
      default: break; }

    if(!fun||!lib) {
      E(0,"unknown tag type \"%s\" (%d)!?\n",key,tag);
      return NOIR_NOTSUPP; }

    *fun=NULL;
    if(*lib) {
      dlclose(*lib);
      *lib=NULL; }

    if(val&&*val&&strcmp(val,"none")) {
      int r=theme_ani_get(t,key,lib,fun);
      if(r!=NOIR_SUCC) {
        E(0,"could not get animation from \"%s\"!?  (%d)\n",val,r);
	return NOIR_FAIL; }}

    return NOIR_SUCC; }

  return NOIR_IGNORED; }



/*****************************************************************************/
/* setup theme */



static int theme_factory(noir_theme *t,char *k,char *x) {
  int   ret=0;
  char *v=NULL;

  switch(*k) {
    case 's':
      v=theme_ms(t,t->path,"engines","sb_",x,".so");
      if(v&&((ret=dlmulti(x,v,RTLD_NOW,&t->sb_lib,
                          "!sb_new !sb_set !sb_dst",
                          &t->sb_new,&t->sb_set,&t->sb_dst))==NOIR_SUCC)) {
        theme_el_set(&t->data,"scrollbarfactory",x);
        ret=PT_SB; }
      break;
    case 'b':
      v=theme_ms(t,t->path,"engines","bg_",x,".so");
      if(v&&(((ret=dlmulti(x,v,RTLD_NOW,&t->bg_lib,
                           "!bg_new !bg_scroll !bg_dst ?bg_upd",
                           &t->bg_new,&t->bg_scr,&t->bg_dst,
                           &t->bg_upd))==NOIR_SUCC)||(ret==NOIR_PARTIAL))) {
        theme_el_set(&t->data,"backgroundfactory",x);
        ret=PT_BG; }
      break;
    case 'r':
      v=theme_ms(t,t->path,"engines","rb_",x,".so");
      if(v&&((ret=dlmulti(x,v,RTLD_NOW,&t->rubber_lib,
                          "!rubber_new !rubber_set !rubber_dst",
                          &t->rubber_new,&t->rubber_set,&t->rubber_dst))==NOIR_SUCC)) {
        theme_el_set(&t->data,"rubberbandfactory",x);
        ret=PT_RB; }
      break;
    case 'i':
      v=theme_ms(t,t->path,"engines","ic_",x,".so");
      if(v&&(((ret=dlmulti(x,v,RTLD_NOW,&t->icon_lib,
                           "!icon_els_create ?icon_set_state",
                           &t->icon_els_create,
                           &t->icon_set_state))==NOIR_SUCC)||(ret==NOIR_PARTIAL))) {
        theme_el_set(&t->data,"iconfactory",x);
        ret=PT_IC; }
      break;
    case 'e': {
      theme_area *a=malloc(sizeof(struct _theme_area));
      icon       *i=NULL;

      if(a) {
        memset(a,0,sizeof(struct _theme_area));
        if((i=malloc(sizeof(struct _icon)))) {
          memset(i,0,sizeof(struct _icon));
          v=theme_ms(t,t->path,"engines","ix_",x,".so");
          if(v&&((ret=dlmulti(x,v,RTLD_NOW,&a->lib,
                              "!ext_upd !ext_dst !ext_evt",
                              &a->ext_upd,&a->ext_dst,&a->ext_evt))==NOIR_SUCC)) {
            ret=PT_IX;
            i->state=IE_DFL;
            i->name=strdup(x);
            i->label=strdup("");
            i->user1=(void *)a;
            i->next=t->areas;
            t->areas=i;
            theme_el_set(&t->data,"extensions",x); }}

        if(!ret) {
          free(a);
          if(i)
            icon_dst(i); }}}
      break;
    case 't':
      switch(k[1]) {
        case 't':
        case 'o':
          v=theme_ms(t,t->path,"engines","tt_",x,".so");
          if(v&&((ret=dlmulti(x,v,RTLD_NOW,&t->tt_lib,
                              "!tt_new !tt_dst",
                              &t->tt_new,&t->tt_dst))==NOIR_SUCC)) {
            theme_el_set(&t->data,"tooltipfactory",x);
            ret=PT_TT; }
          break;
        case 'y':
        case 'b':
          v=theme_ms(t,t->path,"engines","tb_",x,".so");
          if(v&&((ret=dlmulti(x,v,RTLD_NOW,&t->tb_lib,
                              "!tb_redraw !tb_set !tb_dst",
                              &t->tb_redraw,&t->tb_set,&t->tb_dst))==NOIR_SUCC)) {
            theme_el_set(&t->data,"typebufferfactory",x);
            ret=PT_TB; }
          break;
        default:
          E(0,"unknown engine-type \"%s\" ignored.\n",k); }
      break;
    default:
      E(0,"unknown engine-type \"%s\" ignored.\n",k); }

  if(v) {
    if(ret<=0) {
      if(ret!=NOIR_PARTIAL)
        E(0,"dlopen() failed: %s\n",dlerror());
      else
        E(0,"engine %s did not provide all required symbols, ignored...\n",v); }
#if 0
    else
      theme_engine_scan(v);
#endif
    free(v); }

  return ret; }
#endif



static inline ic_mode theme_suss_iconmode(char *v) {
  ic_mode  ret=IC_AUTO;
  char    *p=&v[strlen(v)-1];

  if(!strcmp(v,"auto"))
    ret=IC_AUTO;
  else if(!strcmp(v,"fill"))
    ret=IC_FILL;
  else if(!strcmp(v,"w"))
    ret=IC_W;
  else if(!strcmp(v,"h"))
    ret=IC_H;
  else if(*p=='w')
    ret=IC_WH_W;
  else if(*p=='h')
    ret=IC_WH_H;
  else
    E(0,"unknown icon-mode \"%s\", falling back on \"auto\"...\n",v);
  return ret; }



int theme_parse_line(void *data,char *k,char *v) {
  noir_sess  *ns=(noir_sess *)data;
  noir_theme *t=((noir_theme *)noir_sess_get_theme(ns));
  int         ret=0;
  char       *p,*f;
  size_t      l;

  if(!t||!k||!v||!*k||!*v)
    return 0;

  if((p=strchr(k,':')))
    *p='\0';

  theme_el_canonize_key(k,TRUE);

  l=strlen(k);
  if(!l)
    return 0;

  E(3,"  %s=%s\n",k,v);

#ifdef HAVE_EVAS2
  if((l>7)&&!strncmp(k,"idle",4))
    theme_idle_install(ns,k,v,l);

  else if(((l>6)&&(!strcmp(k+l-7,"factory")||!strcmp(k+l-6,"engine")))||
          !strcmp(k,"extensions"))
    ret=theme_factory(t,k,v);

  else if(!strcmp(k,"shelf")) {
    if((!strcmp(v,"NONE")&&(t->shelf_bg=strdup("")))||
       ((v[0]=='/')&&(t->shelf_bg=strdup(v)))||
       (t->shelf_bg=theme_ms(NULL,t->path,t->name,"",v,".bg.db"))||
       (t->shelf_bg=theme_ms(NULL,t->path,t->name,"",v,".jpg"))||
       (t->shelf_bg=theme_ms(NULL,t->path,t->name,"",v,".png"))||
       (t->shelf_bg=theme_ms(NULL,t->path,t->name,"",v,".eet"))||
       (t->shelf_bg=theme_ms(NULL,t->path,t->name,"",v,""))) {
      theme_el_set(&t->data,k,t->shelf_bg);
      ret=PT_SH; }}

  else if(!strcmp(k,"iconview")) {
    if((!strcmp(v,"NONE")&&(t->iconview_bg=strdup("")))||
       ((v[0]=='/')&&(t->iconview_bg=strdup(v)))||
       (t->iconview_bg=theme_ms(NULL,t->path,t->name,"",v,".bg.db"))||
       (t->iconview_bg=theme_ms(NULL,t->path,t->name,"",v,".jpg"))||
       (t->iconview_bg=theme_ms(NULL,t->path,t->name,"",v,".png"))||
       (t->iconview_bg=theme_ms(NULL,t->path,t->name,"",v,".eet"))||
       (t->iconview_bg=theme_ms(NULL,t->path,t->name,"",v,""))) {
    theme_el_set(&t->data,k,t->iconview_bg);
    ret=PT_IV; }}

  else if(!strcmp(k,"iconset")&&!ns->iconset)
    ns->iconset=strdup(v);

  else if(!strcmp(k,"typebufferfont")||
          !strcmp(k,"tooltipfont")||
          !strcmp(k,"textpathfont")||
          !strcmp(k,"iconfont")) {
    f=theme_font_find(t,v);
    theme_el_set(&t->data,k,f);
    free(f); }

  else if(!strcmp(k,"iconmode"))
    t->icon_mode=theme_suss_iconmode(v);

  else
#endif
  {
/* E(0,"unknown configuration key \"%s\" ignored\n",k); */
    theme_el_set(&t->data,k,v); }

  return ret; }



static int theme_parse(noir_sess *ns) {
  noir_theme *t=((noir_theme *)noir_sess_get_theme(ns));
  int         ret=NOIR_FAIL;
  char       *cfg=NULL;
  int         done=0;
  static int  icon_mapping=0;

  size_t      l=strlen(home)+sizeof(DIR_USER)+sizeof(THEME_CFG)+1;

  if(!strcmp(t->name,"default")&&(cfg=malloc(l))) {
    snprintf(cfg,l,"%s/" DIR_USER "/" THEME_CFG,home);
    E(1,"looking for user-config in \"%s\"...\n",cfg);
    if(!file_exists(cfg)) {
      free(cfg);
      cfg=NULL; }}

  if(!cfg)
    cfg=theme_ms(NULL,t->path,t->name,"",THEME_CFG,"");

  if(cfg) {
    FILE *fh;
    char l[MIME_MAX_LEN];
    char *p,*p3;

    if((fh=fopen(cfg,"r"))) {
      while(fgets(l,MIME_MAX_LEN,fh)) {
        if(parse_line(l,&p,&p3,NULL,NULL)==2)
          done|=theme_parse_line(ns,p,p3); }

      fclose(fh);
      if((PT_ALL&done)!=PT_ALL) {
        char *fact[]={ "scrollbar", "tooltip", "typebuffer", "rubberband", "background", "icon", "shelf", "iconview" };
        int need=PT_ALL-(PT_ALL&done),facts=sizeof(fact)/sizeof(char *),i;
        /* E(0,"setup incomplete, need to set all engines! (got $%x, still need $%x)\n",done,need); */
        for(i=0;i<facts;i++) {
          if(need&(1<<i))
            E(0,"factory MISSING: %s\n",fact[i]);
          else
            E(1,"factory present: %s\n",fact[i]); }}
      else {
        char *is=NULL,*pr=NULL,*v=ns->iconset;

        if(v) {
          if(*v=='/')
            is=strdup(v);
          else if((pr=get_setup_file(DIR_ICONS))) {
            is=g_strconcat(pr,"/",v,NULL);
            free(pr); }}

        if(is&&S_ISDIR(file_exists(is))) {
          ns->iconset=is;
          E(1,"adding path for icon-set \"%s\" {%s}...\n",v,is);
          paths_add(ns->iconset=is); }
        else {
          ns->iconset=NULL;
          E(0,"broken icon-set \"%s\" -- \"%s\" not found or not a directory; trying to fall back to default...\n",v,is?is:"<none>");
          if(is)
            free(is); }

        if(v)
          free(v);

        ret=NOIR_SUCC; }}

    /* this is uncouth. mappings should be per-theme. ### FIXME */
    if(!icon_mapping) {
      if(icon_path_load()!=NOIR_SUCC)
        E(0,"unable to load icons...\n");
      else
	icon_mapping++; }

    free(cfg); }
  else
    E(0,"configuration for theme not found...\n");

  return ret; }



/**
 * theme_init - initialize theme sub-system for a session
 * @ns - the session
 * @cwd - current working-directory
 */
int theme_init(noir_sess *ns,char *cwd) {
  char      *s,*n,*p;
  size_t     l;
  int        ret;

  if(!((noir_theme *)noir_sess_get_theme(ns)))
    return NOIR_FAIL;
  if(!((noir_theme *)noir_sess_get_theme(ns))->name)
    return NOIR_NODATA;

  n=((noir_theme *)noir_sess_get_theme(ns))->name;

  if(n[0]=='/') {
    l=strlen(n)-1;

    if(n[l]=='/')
      n[l--]='\0';
    p=strrchr(n,'/');
    ((noir_theme *)noir_sess_get_theme(ns))->path=n;
    if(p) {
      *(p++)='\0';
      ((noir_theme *)noir_sess_get_theme(ns))->name=p; }}

  else if((p=strrchr(n,'/'))) {
    if(!p[1]) {
      *p='\0';
      p=strrchr(n,'/'); }
    if(!p)
      ((noir_theme *)noir_sess_get_theme(ns))->path=cwd;
    else {
      l=strlen(cwd)+strlen(n)+3;
      *(p++)='\0';
      ((noir_theme *)noir_sess_get_theme(ns))->name=p;
      if((s=malloc(l))) {
        int ret;
        ret=snprintf(s,l,"%s/%s",cwd,n);
        if((ret>=0)&&(ret<=(signed int)l))
          ((noir_theme *)noir_sess_get_theme(ns))->path=s; }}}

  else if(!(((noir_theme *)noir_sess_get_theme(ns))->path=get_setup_file(DIR_THEME)))
    return NOIR_FAIL;

  E(1,"adding path \"%s\" {%s}...\n",((noir_theme *)noir_sess_get_theme(ns))->path,((noir_theme *)noir_sess_get_theme(ns))->name);
  paths_add(((noir_theme *)noir_sess_get_theme(ns))->path);

  E(1,"reading theme from \"%s\" in \"%s\"...\n",((noir_theme *)noir_sess_get_theme(ns))->name,((noir_theme *)noir_sess_get_theme(ns))->path);

  theme_el_listen(&((noir_theme *)noir_sess_get_theme(ns))->data,LISTENER,"openfolderanimation",theme_string_listener,THEME_ODA,ns);
  theme_el_listen(&((noir_theme *)noir_sess_get_theme(ns))->data,LISTENER,"openfileanimation",theme_string_listener,THEME_OFA,ns);
  theme_el_listen(&((noir_theme *)noir_sess_get_theme(ns))->data,LISTENER,"openappanimation",theme_string_listener,THEME_OAA,ns);

  ret=theme_parse(ns);

  return ret; }



/**
 * de-initialize theme sub-system for a single canvas
 * @il  the icon-list
 */
int theme_canvas_destall(icon_list *il) {
  noir_theme *t;

  if(!il||!(t=il->theme))
    return NOIR_FAIL;

  /* have destalled idlers before we removed the views */

  if(t->bg_dst&&il->bg)   { t->bg_dst(&(il->bg));   il->bg=NULL; }
  if(t->sb_dst&&il->sb_v) { t->sb_dst(&(il->sb_v)); il->sb_v=NULL; }
  if(t->sb_dst&&il->sb_h) { t->sb_dst(&(il->sb_h)); il->sb_h=NULL; }

  return NOIR_SUCC; }



/**
 * de-initialize theme sub-system for a session
 * @ns - the session
 */
int theme_exit(noir_sess *ns) {
  noir_theme    *t=((noir_theme *)noir_sess_get_theme(ns));
  noir_theme_el *e;
#ifdef HAVE_EVAS2
  icon          *i;
  theme_area    *a;

  if(t) {
    /* have destalled idlers before we removed the views */
#else
  if(t) {
#endif

    if(t->tb_data)          free(t->tb_data);
    if(t->rubber_data)      free(t->rubber_data);
    if(t->icon_data)        free(t->icon_data);

    if(t->shelf_bg)         free(t->shelf_bg);
    if(t->iconview_bg)      free(t->iconview_bg);
/*  if(t->name)             free(t->name); */

    if(ns->tb)              t->tb_dst(&ns->tb);          /* typebuffer */
    if(ns->tt_evas)         t->tt_dst(&(ns->tt_evas));   /* tooltip */

    /* do not free the following for now; paths_exit() will */
/*  if(t->path)             free(t->path); */

    if(ns->iconset) {
/*    free(ns->iconset); */   /* valgrind alert */
      ns->iconset=NULL; }

    while((e=t->data))
      theme_el_dst(&t->data,e);

#ifdef HAVE_EVAS2
    for(i=t->areas;i;i=i->next) {        /* dest extensions */
      a=i->user1;
      a->ext_dst(i);
      if(a)
        free(a);
      i->user1=NULL; }
    icon_dst(t->areas);
#endif

    memset(t,0,sizeof(noir_theme));
    free(t);
    noir_sess_set_theme(ns,NULL); }
  return NOIR_SUCC; }
