#ifndef __META__
#  define __META__

#if defined(__cplusplus)
extern "C" {
#endif

#  include <time.h>

#  define META_MAX_LEN          4096
#  define TOOLTIP_PROVIDERS    "evidence.providers"

/* deprecated */
#  define meta_meta(a,b)        meta_in_group((a),(b),MY_NAME)
#  define meta_init_str(m,k,d)  meta_dflt((m),META_RW,META_STR,(k),(d),MY_NAME,"",prov)

#  define META_DFLT_GRP        "basic"


typedef enum {
  META_NA,                  /**< meta_set(), meta_chg(): keep old flags */
  META_RO,                  /**< read-only */
  META_RW,                  /**< read/write */
  META_DO,                  /**< read/write */
  META_EF                   /**< entry-field -- has no value, but can generate values */
} meta_flags;

typedef enum {
  META_OLD,                 /**< meta_set(), meta_chg(): keep old type */
  META_STR,                 /**< C-string, in locale-dependent encoding */
  META_UTF8,                /**< C-string, in UTF8-encoding */
  META_ACL,                 /**< Access Control List */
  META_INT,                 /**< integer */
  META_BIN,                 /**< binary data (reserved) */
  META_DBL,                 /**< double-precision floating-point value */
  META_FLT                  /**< floating-point value */
} meta_type;

typedef struct _provider {
  char              *file;  /**< name and path of shared object */
  char              *mime;  /**< same as hash-key, do not free()! */
  void              *lib;   /**< library-handle */
  void              *data;  /**< plug-in specific: returned from init-function (if any), passed to exit-function (if any) */
  time_t             ts;    /**< time-stamp (last use) */
  struct _provider  *next;  /**< next provider in list */
} provider;

typedef struct _meta {
  char         *key;        /**< name of entry */
  char         *grp;        /**< group entry is part of */
  char         *dsc;        /**< description of entry (if any) */
  void         *val,        /**< value, as C-string */
               *prvs;       /**< previous value, as C-string, if any */
  meta_flags    flg;        /**< flags (read-only etc.) */
  meta_type     type;       /**< type (UTF8, integer, ACL, ...) */
  provider     *prov;       /**< provider that created this entry */
  void         *user1;      /**< user-data (application-specific) */
  long          user2;      /**< user-data (application-specific) */
  void         *user3;      /**< user-data (application-specific) */
  struct _meta *next;       /**< next meta-data entry in list */
} meta;

int   meta_chg(meta *,meta_flags,meta_type,const char *,const char *);
int   meta_set(meta **,meta_flags,meta_type,const char *,const char *,const char *,const char *,provider *p);
int   meta_add(meta **,meta_flags,meta_type,const char *,const char *,const char *,const char *,provider *p);
int   meta_del(meta **,char *,char *);
meta *meta_dflt(meta **,meta_flags,meta_type,const char *,const char *,const char *,const char *,provider *);
int   meta_del_all(meta **);
int   meta_del_all_for(meta **,provider *);
int   meta_set_for_file(meta **,char *,provider *);
int   meta_get_for_file(meta **,char *,provider *);
int   meta_update(meta **,char *);
meta *meta_in_group(meta **,const char *,const char *);

int   meta_handle_file(provider *,char *,meta **);

int   meta_lib_gc(void);
void *meta_lib_get(provider *);

char *meta_to_string(meta **);

int   meta_init(void);
int   meta_exit(void);

/** meta_internal - must be implemented by the hosting application! */
int   meta_internal(meta **,char *,char *);

#  ifdef __cplusplus
  }
#  endif

#endif
