#ifndef THUMBNAIL_PRIVATE_H
#  define THUMBNAIL_PRIVATE_H

#  include "config.h"

#  include "thumbnail.h"

#  include <time.h>

#  define EXT_JPG  ".jpg"
#  define EXT_PNG  ".png"
#  define EXT_TTF  ".ttf"



#  define EVIDENCE_THUMB_BASE 128



typedef enum {
  TD_FAIL_ME,
  TD_NORMAL,
  TD_LARGE,
  TD_BASE,
  TD_FAIL } thumbnaildir;



typedef enum {
  TT_DK,
  TT_EPEG,
  TT_PNG,
  TT_FOLDER } thumbnailtype;



struct _thumbnail_plugin {
  void  *lib;
  void  *data;
  char  *file;
  thumbnailer_capabilities can;
  int  (*init)(struct _thumbnail_plugin *);
  int  (*exit)(struct _thumbnail_plugin *);
  int  (*make)(struct _thumbnail_plugin *,Evidence_Thumb *,void **);
    int  (*live)(struct _thumbnail_plugin *,Evidence_Thumb *,thumbnailer_capabilities request);
  struct _thumbnail_plugin *next;
};



struct _Evidence_Thumb {
   void        *theme;          /* the noir_theme */
   char        *uri;            /* URI of original file */
   char        *hashedname;     /* MD5 hash of above URI (no extension) */
   char        *origfile;       /* path to original image */
   char        *thumbfile;      /* path to thumbnail of image */

   struct {                     /* mtime.file is the original's MTime, */
     time_t     file,           /* as per stat().  it is saved in */
                thumb;          /* mtime.thumb (in the thumbnail's tags). */
   } mtime;

   thumbnaildir  size,          /* TD_NORMAL or TD_LARGE */
                 found_in;      /* the dir the thumbnail lives in */
   thumbnailtype type;          /* EPEG, PNG, folder-Image, ... */

   int           w,h,alpha;     /* size of the thumbnail, not the original */
   void         *data;

   struct _thumbnail_plugin *tp;

   struct _Evidence_Thumb *next;
};



struct _thumbnail_plugin *thumbnailer_find_by_mime(const char *mime);
char *evidence_thumbnail_path(thumbnaildir td,const char *file,const char *ext);
char *evidence_get_tmpname(const char *file,const char *ext);



#  ifdef HAVE_IMLIB2
#    define X_DISPLAY_MISSING
#    include <Imlib2.h>
     Imlib_Image evidence_thumb_imlib_standardize(Evidence_Thumb *e);
#  endif
#endif
